#include <MFRC522.h>
#include <LiquidCrystal_I2C.h>
#include <Keypad.h>
#include <Servo.h>
#include <SPI.h>
#include <Wire.h>

MFRC522 mfrc522(10, 9); // MFRC522 mfrc522(SS_PIN, RST_PIN)
LiquidCrystal_I2C lcd(0x27, 16, 2);
Servo sg90;


constexpr uint8_t greenLed = 6;
constexpr uint8_t redLed = 7;
constexpr uint8_t servoPin = 8;

char initial_password[4] = {'1', '2', '3', '4'};  // Variable to store initial password
String tagUID = "53 81 87 14";  // String to store UID of tag
String tag_admin="33 10 1B 15";
char password[4];   // Variable to store users password
boolean RFIDMode = true; // boolean to change modes
boolean AdminMode =false;
char key_pressed = 0; // Variable to store incoming keys
uint8_t i = 0;  // Variable used for counter

// defining how many rows and columns our keypad have
const byte rows = 4;
const byte columns = 4;

// Keypad pin map
char hexaKeys[rows][columns] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

// Initializing pins for keypad
byte row_pins[rows] = {A0, A1, A2, A3};
byte column_pins[columns] = {5, 4, 3,2};

Keypad keypad_key = Keypad( makeKeymap(hexaKeys), row_pins, column_pins, rows, columns);


void setup() {
   
  
  lcd.init();
  lcd.begin(16,2);   // LCD screen
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.clear();
 
  pinMode(redLed, OUTPUT);
  pinMode(greenLed, OUTPUT);
  sg90.attach(servoPin);  //Declare pin 8 for servo
  sg90.write(0); // Set initial position 
  
  SPI.begin();      // Init SPI bus
  mfrc522.PCD_Init();   // Init MFRC522

}

void loop() {
  
  // Look for new cards

    if ( ! mfrc522.PICC_IsNewCardPresent()) {
      lcd.setCursor(0, 0);
  
      RFIDMode=false;;
    }

    // Select one of the cards
    if ( ! mfrc522.PICC_ReadCardSerial()) {
      lcd.setCursor(0, 0);
    if(!AdminMode){
      lcd.print("   LOCKED");
      lcd.setCursor(0, 1);
      lcd.print("TYPE PASS/SCAN");
  }
      RFIDMode=false;
    }
   //Reading from the card
   if(RFIDMode==true)
   {
    String tag = "";
    for (byte j = 0; j < mfrc522.uid.size; j++)
    {
      tag.concat(String(mfrc522.uid.uidByte[j] < 0x10 ? " 0" : " "));
      tag.concat(String(mfrc522.uid.uidByte[j], HEX));
    }
    tag.toUpperCase();
    
    //Checking the card
    if (tag.substring(1) == tagUID)
    {
      // If UID of tag is matched.
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("OPEN");
      sg90.write(90);
      digitalWrite(greenLed, HIGH);
      delay(5000);
      lcd.clear();
      digitalWrite(greenLed, LOW);
      sg90.write(0);
      
    }
  else
     if (tag.substring(1) == tag_admin)
    {
      // If UID of admin_tag is matched.
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("MOD ADMIN");
      delay(2000);
      lcd.clear();
      AdminMode=true;
      digitalWrite(greenLed, HIGH);
      delay(3000);
      digitalWrite(greenLed, LOW);
      tag="";
      
    }

    else
    {
      // If UID of tag is not matched.
   
     
      digitalWrite(redLed, HIGH);
      lcd.clear();
      lcd.print("   LOCKED");
      lcd.setCursor(0, 1);
      lcd.print("BAD TAG");
      delay(3000);
      lcd.clear();
      digitalWrite(redLed, LOW);

    }
  }

  if(AdminMode==true)
  {
    key_pressed = keypad_key.getKey(); // Storing keys
     digitalWrite(greenLed, HIGH);
    lcd.setCursor(0, 0);
    lcd.print("PRESS C OR D");
    lcd.setCursor(0, 1);
     lcd.print("C-ch pass,D-ex");
    if(key_pressed=='C'){
      
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("TYPE NEW PASS:");
      lcd.setCursor(0, 1);
    for(int j=0;j<4;)
        { 
           key_pressed = keypad_key.getKey();
           if(key_pressed){
            
            lcd.print("*");
            password[j]=key_pressed;
            j++;
           }
        }

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("THE PASS IS:");
      lcd.setCursor(0, 1);
      for(int j=0;j<4;j++)lcd.print(password[j]);
      delay(3000);
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.clear();
      
      strncpy(initial_password,password,4);
      key_pressed=' ';
    }
    
     
    if(key_pressed=='D'){
    digitalWrite(greenLed, LOW);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("ADMIN OUT");
    delay(2000);
    lcd.clear();
    AdminMode=false;
    delay(3000);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("LOCKED");
    }
  }
  else 
  if (RFIDMode == false) {
    key_pressed = keypad_key.getKey(); // Storing keys
    if (key_pressed)
    { password[i++] = key_pressed; // Storing in password variable
      lcd.clear();
     lcd.setCursor(1, 0);
      for(int j=0;j<i;j++)
         lcd.print("*");
      delay(500);
     
    }
    if (i == 4) // If 4 keys are completed
    { 
   
      delay(1000);
      if (!(strncmp(password, initial_password, 4))) // If password is matched
      {  
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("OPEN");
      sg90.write(90);
      digitalWrite(greenLed, HIGH);
      delay(5000);
      sg90.write(0);
      digitalWrite(greenLed, LOW);
       
      }
      else    // If password is not matched
      { 
        
        digitalWrite(redLed, HIGH);
        lcd.setCursor(0, 1);
        for(int j=0;j<i;j++)
        lcd.print(password[j]);
        lcd.print(" BAD PASS");
        delay(3000);
        lcd.clear();
        digitalWrite(redLed, LOW);
     
      }
      i=0;
    }
  }
  RFIDMode=true;
}
